clc
clear

% %MINIMIZATION ROUTINE FOR HIDE AND SEEK:
% %---------------------------------------

% %Check for the right model in LogLikeHS and the right data (aggregate or one of the treatments)

% %Parameters to be estimated: [pH,pS,qH,qS,r,s,t,u,e,eH,eS,fH,fS,lambda]
% %----------------------------------------------------------------------

%pH,pS: center/end bias for Hiders and Seekers
%qH,qS: focally labeled bias Hiders and Seekers
%r: L0 type frequency
%s: L1 type frequency
%t: L2 type frequency
%u: L3 type frequency
%e: error rate

%eH,eS: payoff perturbation for chosing end locations
%fH,fS: payoff perturbation for chosing focally the labeled action.

% %--------
% %LEVEL-K:
% %--------

% %RANDOM INITIAL VALUES, EVALUATION OF THE FUNCTION;
% %-------------------------------------------------%
% %When asymmetric L0
% %------------------
%NOTE: r=0 so p and q only matter for the region. Adjust and replace the "fix the region" below
%In the asymmetric, different regions can be chosen for L0 Hiders and L0 Seekers.
%(a)Pick and fix the initial value that belongs to a region in the random search
%(b)Change accordingly the weak Equality Restrictions in the fmincon: 
%NOTE: We do not need a different model for symmetric L0, just fix the region such that pH=pS and 
%qH=qS.

%REGION 1: 3p+2q>2, p+2q>1 2q>p
%I.e.: p=0.3 q=0.6

%REGION 2: 3p+2q>2, p+2q>1 2q<p
%I.e.: p=0.6 q=0.25

%REGION 3: 3p+2q>2, p+2q<1 2q<p
%I.e.: p=0.7 q=0.1

%REGION 4: 3p+2q<2, p+2q<1 2q<p
%I.e.: p=0.4 q=0.15

%REGION 5: 3p+2q<2, p+2q<1 2q>p
%I.e.: p=0.1 q=0.3

%REGION 6: 3p+2q<2, p+2q>1 2q>p
%I.e.: p=0.1 q=0.6

minfv=Inf*ones(100,1);

for e=1:1000;

initialpH="fix the region";
initialqH="fix the region";
initialpS="fix the region";
initialqS="fix the region";
initialr=0;
initials=(1-initialr)*rand(1);
initialt=(1-initials-initialr)*rand(1);
initialu=(1-initials-initialr-initialt)*rand(1);
initiale=rand(1);

initialvalues(:,e)=[ initialpH; initialqH;...
                     initialpS; initialqS;... 
                     initialr; initials; initialt; initialu;...
                     initiale];
feval(e,1)=LogLikeHS(initialvalues(:,e));
end;

minfv=min(feval);
mininitialval=[0;0;0;0;0;0;0;0;0];
minfeval=0;

for e=1:1000;
if minfv==feval(e,1);
mininitialval=initialvalues(:,e);
minfeval=feval(e,1);
else;
mininitialval=mininitialval;
minfeval=minfeval;
end;
end;

feval;
minfeval
mininitialval

[U, fval, exitflag, output]=fmincon('LogLikeHS', mininitialval, ...
[1 1 0 0 0 0 0 0 0; 0 0 1 1 0 0 0 0 0; 0 0 0 0 0 1 1 1 0], [1; 1; 1], [1 0 0 0 0 0 0 0 0; 0 1 0 0 0 0 0 0 0; 0 0 1 0 0 0 0 0 0; 0 0 0 1 0 0 0 0 0; 0 0 0 0 1 0 0 0 0],...
["fix the region"; "fix the region"; "fix the region"; "fix the region"; 0], [0; 0; 0; 0; 0; 0; 0; 0; 0], [1; 1; 1; 1; 1; 1; 1; 1; 1], [], optimset('TolFun', 1.0000e-100, 'MaxFunEvals',5000, 'MaxIter', 4000,'MaxFunEvals', 100000))

% %When symmetric L0
% %-----------------
%NOTE: r=0 so p and q only matter for the region. Adjust and replace the "fix the region" below.
%(a)Pick and fix the initial value that belongs to a region in the random search
%(b)Change accordingly the Equality Restrictions in the fmincon: 

%REGION 1: 3p+2q>2, p+2q>1 2q>p
%I.e.: p=0.3 q=0.6

%REGION 2: 3p+2q>2, p+2q>1 2q<p
%I.e.: p=0.6 q=0.25

%REGION 3: 3p+2q>2, p+2q<1 2q<p
%I.e.: p=0.7 q=0.1

%REGION 4: 3p+2q<2, p+2q<1 2q<p
%I.e.: p=0.4 q=0.15

%REGION 5: 3p+2q<2, p+2q<1 2q>p
%I.e.: p=0.1 q=0.3

%REGION 6: 3p+2q<2, p+2q>1 2q>p
%I.e.: p=0.1 q=0.6

% minfv=Inf*ones(10000,1);
% 
% for e=1:10000;
% 
% initialp="fix the region";
% initialq="fix the region";
% initialr=0;
% initials=(1-initialr)*rand(1);
% initialt=(1-initials-initialr)*rand(1);
% initialu=(1-initials-initialr-initialt)*rand(1);
% initiale=rand(1);
% 
% initialvalues(:,e)=[ initialp; initialq;...
%                      initialr; initials; initialt; initialu;... 
%                      initiale];
% 
% feval(e,1)=LogLikeHS(initialvalues(:,e));
% end;
% 
% minfv=min(feval);
% mininitialval=[0;0;0;0;0;0;0];
% minfeval=0;
% 
% for e=1:10000;
% if minfv==feval(e,1);
% mininitialval=initialvalues(:,e);
% minfeval=feval(e,1);
% else;
% mininitialval=mininitialval;
% minfeval=minfeval;
% end;
% end;
% 
% feval;
% minfeval
% mininitialval
% 
% 
% [U, fval, exitflag, output]=fmincon('LogLikeHS', mininitialval, ...
% [1 1 0 0 0 0 0; 0 0 1 1 1 1 0], [1; 1], [1 0 0 0 0 0 0; 0 1 0 0 0 0 0; 0 0 1 0 0 0 0], ["fix the region"; "fix the region"; 0],...
% [0; 0; 0; 0; 0; 0; 0], [1; 1; 1; 1; 1; 1; 1], [], optimset('TolFun', 1.0000e-100, 'MaxFunEvals',5000, 'MaxIter', 4000,'MaxFunEvals', 100000))

% %EQUILIBRIUM WITH PERTURBATION
% %-----------------------------
%Add equality restrictions of eH=eS and fH=fS for the case of same magnitudes:
%[1 0 -1 0; 0 1 0 -1], [0; 0]

% minfv=Inf*ones(10000,1);
% 
% for e=1:10000;
% 
% initialeH=.5*rand(1);
% initialfH=.3*rand(1);
% initialeS=.5*rand(1);
% initialfS=.3*rand(1);
% 
% initialvalues(:,e)=[ initialeH; initialfH; initialeS; initialfS];
% feval(e,1)=LogLikeHS(initialvalues(:,e));
% minfv=min(feval);
% 
% if minfv==feval(e,1);
% mininitialvalues=initialvalues(:,e);
% minfeval=feval(e,1);
% else;
% mininitialvalues=mininitialvalues;
% minfeval=minfeval;
% end;
% end;
% 
% feval;
% minfeval
% mininitialvalues
% 
% [U, fval, exitflag, output]=fmincon('LogLikeHS', mininitialvalues,...
% [-2 1 0 0 ; 2 1 0 0 ; 2 -3 0 0 ; 0 0 -2 1 ; 0 0 2 1 ; 0 0 2 -3 ],[3; 3; 3; 3; 3; 3],...
% [], [], [0; 0; 0; 0], [.5; .5; .5; .5], [], [])